/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.ecf.example.sc.ui.marker;


/* Imports */
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * DataEntryDialog class uses <code>org.eclipse.swt</code> 
 * libraries to implement a dialog that accepts basic personal information that
 * is added to a <code>Table</code> widget or edits a <code>TableItem</code> entry 
 * to represent the entered data.
 */
public class DataEntryDialog {

	
	
	Shell shell;
	String[] values;
	String[] labels;

	private IFile file;

	
	private Text textDate;

	private Text textLast;

	private Text textLanguage;

	private Text textTitle;

	private Text textCreator;

	private Text textSubjectKeywords;

	private Text textDescription;

	private Text textResourceIdentifier;
	
public DataEntryDialog(Shell parent) {
	shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.PRIMARY_MODAL);
	shell.setLayout(new GridLayout());		
}

private void addTextListener(final Text text) {
	text.addModifyListener(new ModifyListener() {
		public void modifyText(ModifyEvent e){
			Integer index = (Integer)(text.getData("index"));
			values[index.intValue()] = text.getText();
		}
	});
}


public IMarker createMarker(IResource resource) {
	
	IMarker marker = null;
    try {
       marker = resource.createMarker("org.eclipse.ecf.example.sc.marker");
       
       marker.setAttribute("Data_create" , textDate.getText());
       marker.setAttribute("Last_update" , textLast.getText());
       marker.setAttribute("Language" , textLanguage.getText());
       marker.setAttribute("Title" , textTitle.getText());
       marker.setAttribute("Creator" , textCreator.getText());
       marker.setAttribute("Subject_and_Keywords" , textSubjectKeywords.getText());
       marker.setAttribute("Description" , textDescription.getText());
       marker.setAttribute("Resource_Identifier" , textResourceIdentifier.getText());
       
       
       
       
    } catch (CoreException e) {
    	//TODO Tratar log
    	
    	 //ErrorDialog.openError(Activator.getSite().getShell(), Messages.getString("errorModifyingTask"), null, e.getStatus());

    }
	return marker;
}

public IMarker[] findMarkers(IResource target) {
    String type = "org.eclipse.ecf.example.sc.marker";
    IMarker[] markers = null;
	try {
		markers = target.findMarkers(type, false, IResource.DEPTH_ZERO);
	} catch (CoreException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	return markers;
 }

private void createControlButtons() {
	Composite composite = new Composite(shell, SWT.NONE);
	composite.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER));
	GridLayout layout = new GridLayout();
	layout.numColumns = 2;
	composite.setLayout(layout);
	
	Button okButton = new Button(composite, SWT.PUSH);
	okButton.setText("Ok");
	okButton.addSelectionListener(new SelectionAdapter() {
		public void widgetSelected(SelectionEvent e) {
			
			IMarker []m = findMarkers(file);
			
			if(m[0].exists())
				try {
					m[0].delete();
				} catch (CoreException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			createMarker(file);
			
			
			
			shell.close();
		}
	});
	
	Button cancelButton = new Button(composite, SWT.PUSH);
	cancelButton.setText("Cancel");
	cancelButton.addSelectionListener(new SelectionAdapter() {
		public void widgetSelected(SelectionEvent e) {
		
			shell.close();
		}
	});
	
	shell.setDefaultButton(okButton);
}

//TODO Precisa melhor essa criacao de componentes
private void createTextWidgets() {
	if (labels == null) return;
	
	
	Composite composite = new Composite(shell, SWT.NONE);
	composite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	GridLayout layout= new GridLayout();
	layout.numColumns = 2;
	composite.setLayout(layout);
	
	
	IMarker[] m = findMarkers(file);

	Label labelDate = new Label(composite, SWT.RIGHT);
	labelDate.setText("Data create:" );	
	textDate = new Text(composite, SWT.BORDER);
	GridData gridData = new GridData();
	gridData.widthHint = 100;
	textDate.setLayoutData(gridData);
	textDate.setText(m[0].getAttribute("Data_create", ""));
	addTextListener(textDate);	

	Label labelLast = new Label(composite, SWT.RIGHT);
	labelLast.setText("Last update:" );	
	textLast = new Text(composite, SWT.BORDER);
	GridData gridDataLast = new GridData();
	gridDataLast.widthHint = 100;
	textLast.setLayoutData(gridDataLast);
	textLast.setText(m[0].getAttribute("Last_update", ""));
	addTextListener(textLast);	

	Label labelLanguage = new Label(composite, SWT.RIGHT);
	labelLanguage.setText("Language:" );	
	textLanguage = new Text(composite, SWT.BORDER);
	GridData gridDataLanguage = new GridData();
	gridDataLanguage.widthHint = 100;
	textLanguage.setLayoutData(gridDataLanguage);
	textLanguage.setText(m[0].getAttribute("Language", ""));
	addTextListener(textLanguage);	

	Label labelTitle = new Label(composite, SWT.RIGHT);
	labelTitle.setText("Title:" );	
	textTitle = new Text(composite, SWT.BORDER);
	GridData gridDataTitle = new GridData();
	gridDataTitle.widthHint = 300;
	textTitle.setLayoutData(gridDataTitle);
	textTitle.setText(m[0].getAttribute("Title", ""));
	addTextListener(textTitle);	

	Label labelCreator = new Label(composite, SWT.RIGHT);
	labelCreator.setText("Creator:" );	
	textCreator = new Text(composite, SWT.BORDER);
	GridData gridDataCreator = new GridData();
	gridDataCreator.widthHint = 100;
	textCreator.setLayoutData(gridDataCreator);
	textCreator.setText(m[0].getAttribute("Creator", ""));
	addTextListener(textCreator);	

	Label labelSubjectKeywords = new Label(composite, SWT.RIGHT);
	labelSubjectKeywords.setText("Subject and Keywords:" );	
	textSubjectKeywords = new Text(composite, SWT.BORDER);
	GridData gridDataSubjectKeywords = new GridData();
	gridDataSubjectKeywords.widthHint = 300;
	textSubjectKeywords.setLayoutData(gridDataSubjectKeywords);
	textSubjectKeywords.setText(m[0].getAttribute("Subject_and_Keywords", ""));
	addTextListener(textSubjectKeywords);	

	Label labelDescription = new Label(composite, SWT.RIGHT);
	labelDescription.setText("Description:" );	
	textDescription = new Text(composite, SWT.BORDER);
	GridData gridDataDescription = new GridData();
	gridDataDescription.widthHint = 300;
	textDescription.setLayoutData(gridDataDescription);
	textDescription.setText(m[0].getAttribute("Description", ""));
	addTextListener(textDescription);	

	Label labelResourceIdentifier = new Label(composite, SWT.RIGHT);
	labelResourceIdentifier.setText("Resource Identifier:" );	
	textResourceIdentifier = new Text(composite, SWT.BORDER);
	GridData gridDataResourceIdentifier = new GridData();
	gridDataResourceIdentifier.widthHint = 200;
	textResourceIdentifier.setLayoutData(gridDataResourceIdentifier);
	textResourceIdentifier.setText(m[0].getAttribute("Resource_Identifier", ""));
	addTextListener(textResourceIdentifier);	
	

}


public String getTitle() {
	return shell.getText();
}


/** 
 * Opens the dialog in the given state.  Sets <code>Text</code> widget contents 
 * and dialog behaviour accordingly.
 *
 * @param 	dialogState	int
 *					The state the dialog should be opened in.
 */
public void open() {
	createTextWidgets();
	createControlButtons();
	shell.pack();
	shell.open();
	Display display = shell.getDisplay();
	while(!shell.isDisposed()){
		if(!display.readAndDispatch())
			display.sleep();
	}
	
}

public void setLabels(String[] labels) {
	this.labels = labels;
}
public void setTitle(String title) {
	shell.setText(title);
}


public void setFileMarker(IFile file) {
	this.file = file;
	
}
}
