package org.eclipse.ecf.example.sc.ui.search;




import org.eclipse.ecf.datashare.IChannelListener;
import org.eclipse.ecf.example.sc.actions.RequestAction;
import org.eclipse.ecf.example.sc.core.SCUIMessages;
import org.eclipse.ecf.example.sc.core.model.Input;
import org.eclipse.ecf.example.sc.exception.ShareCodeException;
import org.eclipse.ecf.example.sc.message.RequestMessage;
import org.eclipse.ecf.example.sc.message.SharedCodeMessage;
import org.eclipse.ecf.example.sc.util.GeneratorUUID;
import org.eclipse.ecf.example.sc.views.ShareCodeResultView;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.DialogPage;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.search.ui.ISearchPage;
import org.eclipse.search.ui.ISearchPageContainer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;



public class ShareCodeSearchPage extends DialogPage implements ISearchPage{

	private static final int COMMENTS = 3;

	private static final int CLASSNAME = 2;

	private static final int CODEBLOCK = 1;

	private static final int IMPORTS = 0;

	/*
	 * 
	 * ee
	class QueryData {
		public String text;
		public boolean isCaseSensitive;
		public int searchElement;
		public int limit;
		public int externalScope;
		public int workspaceScope;
		
		public boolean equals(Object obj) {
			if (obj instanceof QueryData) {
				if (((QueryData)obj).text.equals(text))
					return true;
			}
			return false;
		}
		
	}
	*/

	
	
	private ISearchPageContainer container;
	
	private Button[] limitToButtons = new Button[3];
	private Combo patternCombo;
	private Button[] searchForButtons = new Button[4];

	@SuppressWarnings("unused")
	private IChannelListener presenceChannelListener;

	private Combo comboLicences;

	private Text data;

	public void createControl(Composite parent)  {
		
		Composite result = new Composite(parent,SWT.NONE);
		GridLayout layout = new GridLayout(1, true);
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		result.setLayout(layout);
		result.setLayoutData(new GridData(GridData.FILL_HORIZONTAL|GridData.GRAB_HORIZONTAL));
		
		createPatternSection(result);
		createSettingsSection(result);
		
		hookListeners();	
		
		setControl(result);
		Dialog.applyDialogFont(result);
		//WorkbenchHelp.setHelp(result, IHelpContextIds.SEARCH_PAGE);
	}
		
	
		
	private void createPatternSection(Composite parent) {
		Composite result = new Composite(parent, SWT.NONE);
		result.setLayout(new GridLayout(2, false));
		result.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Label label = new Label(result, SWT.NONE);
		GridData data = new GridData();
		data.horizontalSpan = 2;
		label.setLayoutData(data);
		label.setText(SCUIMessages.getString("sc.txt.expression"));

		patternCombo = new Combo(result, SWT.BORDER);
		patternCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		//caseSensitive = new Button(result, SWT.CHECK);
		//caseSensitive.setText(KEY_CASE_SENSITIVE);
	}
	
	private void createSettingsSection(Composite parent) {
		Composite result = new Composite(parent, SWT.NONE);
		result.setLayout(new GridLayout(2, true));
		result.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Group group = new Group(result, SWT.NONE);
		group.setLayout(new GridLayout(1, true));
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		group.setText(SCUIMessages.getString("sc.txt.searchFor"));
		
		
		searchForButtons[IMPORTS] = new Button(group, SWT.CHECK);
		searchForButtons[IMPORTS].setText(SCUIMessages.getString("sc.ckb.imports"));
		searchForButtons[IMPORTS].setSelection(true);
		
		searchForButtons[CODEBLOCK] = new Button(group, SWT.CHECK);
		searchForButtons[CODEBLOCK].setText(SCUIMessages.getString("sc.ckb.codeBlock"));
		searchForButtons[CODEBLOCK].setSelection(true);
		
		searchForButtons[CLASSNAME] = new Button(group, SWT.CHECK);
		searchForButtons[CLASSNAME].setText(SCUIMessages.getString("sc.ckb.className"));
		searchForButtons[CLASSNAME].setSelection(true);
		
		searchForButtons[COMMENTS] = new Button(group, SWT.CHECK);
		searchForButtons[COMMENTS].setText(SCUIMessages.getString("sc.ckb.javadoc"));
		searchForButtons[COMMENTS].setSelection(true);
		
		
		
		Group group2 = new Group(result, SWT.NONE);
		group2.setLayout(new GridLayout(1, true));
		group2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		group2.setText(SCUIMessages.getString("sc.txt.limitTo"));
		
		comboLicences = new Combo(group2, SWT.BORDER );
		//add the value catch in the properties and adds in comboBox 
		int comboLength = Integer.parseInt(SCUIMessages.getString("sc.cmd.lenght.licenses"));
		String combo = "sc.cmb.licenses.";
		
		for (int i = 0; i < comboLength; i++) {
			comboLicences.add(SCUIMessages.getString(combo + i));
		}
		
		comboLicences.select(0);
		
		Group group3 = new Group(group2, SWT.NONE);
		group3.setLayout(new GridLayout(2, false));
		group3.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Label label = new Label(group3, SWT.NONE);
		//GridData gdata = new GridData();
		//gdata.horizontalSpan = 2;
		//label.setLayoutData(gdata);
		label.setText(SCUIMessages.getString("sc.txt.classDateCreation"));
		data = new Text(group3, SWT.BORDER);
		data.setSize(80, 20);
		
		/*
		Group group4 = new Group(result, SWT.NONE);
		group4.setLayout(new GridLayout(1, true));
		group4.setLayoutData(new GridData(GridData.FILL_BOTH));

		group4.setText("Remote peer scope");
		searchForButtons[1] = new Button(group4, SWT.RADIO);
		searchForButtons[1].setText("Permitted access only");
		searchForButtons[1].setSelection(true);
		*/

	}
	/*
	
	private IFile findManifestFile(Object item) {
		if (item instanceof IJavaProject)
			item = ((IJavaProject)item).getProject();
			
		if (item instanceof IProject) {
			IFile file = ((IProject) item).getFile("plugin.xml"); //$NON-NLS-1$
			if (file.exists())
				return file;
			file = ((IProject) item).getFile("fragment.xml"); //$NON-NLS-1$
			if (file.exists())
				return file;
		} else if (item instanceof IFile) {
			IFile file = (IFile)item;
			if (file.getName().equals("plugin.xml") //$NON-NLS-1$
				return file;
			}
		}
		return null;
	}
	
	private int getExternalScope() {
		if (externalScopeButtons[0].getSelection())
			return PluginSearchScope.EXTERNAL_SCOPE_ALL;
		if (externalScopeButtons[1].getSelection())
			return PluginSearchScope.EXTERNAL_SCOPE_ENABLED;
		return PluginSearchScope.EXTERNAL_SCOPE_NONE;
	}
	
	private PluginSearchInput getInput() {
		PluginSearchScope scope =
			new PluginSearchScope(
				getWorkspaceScope(),
				getExternalScope(),
				getSelectedResources());

		PluginSearchInput input = new PluginSearchInput();
		int searchFor = getSearchFor();
		input.setSearchElement(searchFor);
		input.setSearchLimit(getLimitTo());
		input.setSearchScope(scope);
		String searchString = patternCombo.getText().trim();
		if (searchFor == PluginSearchInput.ELEMENT_EXTENSION_POINT
			&& searchString.indexOf('.') == -1) {
			searchString = "*." + searchString; //$NON-NLS-1$
		}
		input.setSearchString(searchString);
		input.setCaseSensitive(caseSensitive.getSelection());
		return input;
	}
	
	private int getLimitTo() {
		if (limitToButtons[0].getSelection())
			return PluginSearchInput.LIMIT_DECLARATIONS;
		if (limitToButtons[1].getSelection())
			return PluginSearchInput.LIMIT_REFERENCES;
		return PluginSearchInput.LIMIT_ALL;
	}
	
	private int getSearchFor() {
		if (searchForButtons[0].getSelection())
			return PluginSearchInput.ELEMENT_PLUGIN;
		if (searchForButtons[1].getSelection())
			return PluginSearchInput.ELEMENT_FRAGMENT;
		return PluginSearchInput.ELEMENT_EXTENSION_POINT;
	}
	
	private HashSet getSelectedResources() {
		HashSet result = new HashSet();
		int scope = container.getSelectedScope();
		if (scope == ISearchPageContainer.WORKSPACE_SCOPE)
			return null;
		if (scope == ISearchPageContainer.SELECTION_SCOPE) {
			if (container.getSelection() instanceof IStructuredSelection) {
				IStructuredSelection selection =
					(IStructuredSelection) container.getSelection();
				Iterator iter = selection.iterator();
				while (iter.hasNext()) {
					IFile file = findManifestFile(iter.next());
					if (file != null)
						result.add(file);
				}
			}
		} else if (scope == ISearchPageContainer.WORKING_SET_SCOPE) {
			IWorkingSet[] workingSets = container.getSelectedWorkingSets();
			if (workingSets != null) {
				for (int i = 0; i < workingSets.length; i++) {
					IAdaptable[] elements = workingSets[i].getElements();
					for (int j = 0; j < elements.length; j++) {
						IFile file = findManifestFile(elements[j]);
						if (file != null)
							result.add(file);
					}
				}
			}
		}
		return result;
	}
	
	private int getWorkspaceScope() {
		switch(container.getSelectedScope()) {
			case ISearchPageContainer.SELECTION_SCOPE:
				return PluginSearchScope.SCOPE_SELECTION;
			case ISearchPageContainer.WORKING_SET_SCOPE:
				return PluginSearchScope.SCOPE_WORKING_SETS;
			default:
				return PluginSearchScope.SCOPE_WORKSPACE;
		}
	}
	*/	
	private void hookListeners() {
		searchForButtons[1].addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				boolean selected = searchForButtons[1].getSelection();
				if (selected) {
					limitToButtons[0].setSelection(true);
					limitToButtons[1].setSelection(false);
					limitToButtons[2].setSelection(false);
				}
				limitToButtons[1].setEnabled(!selected);
				limitToButtons[2].setEnabled(!selected);
			} 
		});
		
		/*
		patternCombo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				int index = previousQueries.size() - patternCombo.getSelectionIndex() - 1;
				QueryData data = (QueryData)previousQueries.get(index);
				resetPage(data);
				container.setPerformActionEnabled(patternCombo.getText().length() > 0);
			}
		});
		*/
		patternCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				container.setPerformActionEnabled(patternCombo.getText().trim().length() > 0);
			}
		});	
	}
	
	public boolean performAction() {
		/*
		 * Adiciona somente o content a mensagem.
		 */
		String expression = patternCombo.getText();
		if (expression==null) {
			
		}
		Input input = new Input();
		input.setExpression(expression);
		//input.setDate(date);
		input.setLicense(comboLicences.getText());
		input.setFlagClass(searchForButtons[CLASSNAME].getSelection());
		input.setFlagCodeBlock(searchForButtons[CODEBLOCK].getSelection());
		input.setFlagComments(searchForButtons[COMMENTS].getSelection());
		input.setFlagImport(searchForButtons[IMPORTS].getSelection());

		//TODO sera que precisamos tb mostrar os arquivos locais
		/*
		List retorno = SearchUtil.find(input,  new IProgressMonitor(){
			public void beginTask(String name, int totalWork) {	}
			public void done() {}
			public void internalWorked(double work) {}
			public boolean isCanceled() {return false;}
			public void setCanceled(boolean value) {}
			public void setTaskName(String name) {}
			public void subTask(String name) {}
			public void worked(int work) {}
			});		
		
		
		*/
		
		
		try {
			SharedCodeMessage requestMessage = new RequestMessage();
	//		 TODO popular a mensagem com informacoes para o replier
			requestMessage.setTime(System.currentTimeMillis());
			
			//add a ID to the message
			requestMessage.setMessageId(GeneratorUUID.getInstance().geraUUID());
			requestMessage.setContent(input);
			
			ShareCodeResultView sv = ShareCodeResultView.getInstance(requestMessage.getMessageId());
			//sv.show(retorno);
			
			RequestAction action = new RequestAction();
		
		
			action.execute(requestMessage);
		} catch (ShareCodeException e) {
			MessageDialog.openInformation(this.getShell(),"Shared Code Plug'in", e.getMessage());
		}
		
		return true;
		
	}

	public void setContainer(ISearchPageContainer container) {
		// TODO Identificar a necessidade desse metodo
		this.container = container;
		
	}
	
	/*
	private void resetPage (QueryData data) {
		caseSensitive.setSelection(data.isCaseSensitive);
		
		searchForButtons[0].setSelection(data.searchElement == PluginSearchInput.ELEMENT_PLUGIN);
		searchForButtons[1].setSelection(data.searchElement == PluginSearchInput.ELEMENT_FRAGMENT);
		searchForButtons[2].setSelection(data.searchElement == PluginSearchInput.ELEMENT_EXTENSION_POINT);

		limitToButtons[0].setSelection(data.limit == PluginSearchInput.LIMIT_DECLARATIONS);
		limitToButtons[1].setSelection(data.limit == PluginSearchInput.LIMIT_REFERENCES);
		limitToButtons[1].setEnabled(!searchForButtons[1].getSelection());
		limitToButtons[2].setSelection(data.limit == PluginSearchInput.LIMIT_ALL);
		limitToButtons[2].setEnabled(!searchForButtons[1].getSelection());
		
		externalScopeButtons[0].setSelection(data.externalScope == PluginSearchScope.EXTERNAL_SCOPE_ALL);
		externalScopeButtons[1].setSelection(data.externalScope == PluginSearchScope.EXTERNAL_SCOPE_ENABLED);
		externalScopeButtons[2].setSelection(data.externalScope == PluginSearchScope.EXTERNAL_SCOPE_NONE);
		
		container.setSelectedScope(data.workspaceScope);
		if (data.workingSets != null)
			container.setSelectedWorkingSets(data.workingSets);		
	}
		
	private void saveQueryData() {
		QueryData data = new QueryData();
		data.text = patternCombo.getText();
		data.isCaseSensitive = caseSensitive.getSelection();
		data.searchElement = getSearchFor();
		data.limit = getLimitTo();
		data.externalScope = getExternalScope();
		data.workspaceScope = container.getSelectedScope();
		data.workingSets = container.getSelectedWorkingSets();
		
		if (previousQueries.contains(data))
			previousQueries.remove(data);
			
		previousQueries.add(data);			
		if (previousQueries.size() > 10)
			previousQueries.remove(0);
	}
	
	public void setContainer(ISearchPageContainer container) {
		this.container = container;
	}
	
	public void setVisible(boolean visible) {
		if (visible && patternCombo != null) {
			if (firstTime) {
				firstTime = false;
				String[] patterns = new String[previousQueries.size()];
				for (int i = previousQueries.size() - 1, j = 0;
					i >= 0;
					i--, j++) {
					patterns[j] = ((QueryData) previousQueries.get(i)).text;
				}
	
				patternCombo.setItems(patterns);
				container.setPerformActionEnabled(
					patternCombo.getText().length() > 0);
			}
			patternCombo.setFocus();
		}
		super.setVisible(visible);
	}

	
	*/
	/*
	private ISearchPageContainer container;
	
	
	public void createControl(Composite parent) {
		
		
		
		Composite result = new Composite(parent,SWT.NONE);
		
		RowLayout rowLayout = new RowLayout();
		rowLayout.wrap = false;
		rowLayout.pack = false;
		rowLayout.justify = true;
		rowLayout.type = SWT.HORIZONTAL;
		rowLayout.marginLeft = 5;
		rowLayout.marginTop = 5;
		rowLayout.marginRight = 5;
		rowLayout.marginBottom = 5;
		rowLayout.spacing = 0;
		  
		result.setLayout(rowLayout);
		//result.setLayoutData(new GridData(GridData.FILL_HORIZONTAL|GridData.GRAB_HORIZONTAL));

	    // Create a label to display what the user typed in
	    Label label = new Label(result, SWT.PUSH);
	    label.setText("Search: ");
	    Text text = new Text(result, SWT.PUSH);
	    
	    text.setLayoutData(new RowData(250, 40));
	    
	    // Create the button to launch the error dialog
	    Image image = new Image(result.getDisplay(),getClass().getResourceAsStream("search-button-icon.png"));
	    Button show = new Button(result, SWT.PUSH);
	    show.setImage(image);
	    show.setLayoutData(new RowData(20, 20));
	   
	    
	    		
		//createPatternSection(result);
		//createSettingsSection(result);
		
		//hookListeners();	
		
		setControl(result);
		Dialog.applyDialogFont(result);
		//WorkbenchHelp.setHelp(result, IHelpContextIds.SEARCH_PAGE);
	    

		
	}

	public boolean performAction() {
		
		return false;
	}

	public void setContainer(ISearchPageContainer container) {
		this.container = container;
		
		
	}
*/
}